//
//  MainLayer.m
//
//  Created by Kansuke Nishida on 12/07/25.
//  Copyright 2012年 tonosamart. All rights reserved.
//

#import "MainLayer.h"
#import "SimpleAudioEngine.h"
#import "GameData.h"
#import "TonoCocos.h"
#import "Ninja.h"

#define COUNT_RATE 1.0f

enum {
    OBJECT_NULL,
    OBJECT_GAME
};

enum {
    SPRITE_GAME_NULL,
    SPRITE_GAME_YUKA,
    SPRITE_GAME_LINE,
    SPRITE_GAME_NINJYA,
    SPRITE_GAME_PARTICLE,
    SPRITE_GAME_ENEMY,
    SPRITE_GAME_SHOT,
    SPRITE_GAME_INFO,
    SPRITE_GAME_TEXT
};

@implementation MainLayer

- (id)init{
    self = [super init];
    if (self) {
        // イベントの無効/有効
        self.isTouchEnabled = NO;
        self.isAccelerometerEnabled = NO;
        
        // 初期化
        self.mode = MAIN_LAYER_MODE_INIT;
        self.isEndGame = NO;
        
        [[SimpleAudioEngine sharedEngine] preloadEffect:@"xmiss.caf"];
        [[SimpleAudioEngine sharedEngine] preloadEffect:@"xout.caf"];
    
        // レイヤーの初期化
        [self initLayer];
        [self schedule:@selector(tick:)];
    }
    return self;
}

-(void)tick:(ccTime)dt{
    if(self.mode == MAIN_LAYER_MODE_PLAY){
        [self gamePlay:dt];
    }
}

-(void)dealloc{
    [_ninja release];
    [super dealloc];
}
#pragma mark private

-(void)initLayer{
    [[CCSpriteFrameCache sharedSpriteFrameCache] addSpriteFramesWithFile:@"NInvadersResource.plist"];
    CCSpriteBatchNode *gameBatchNode = [CCSpriteBatchNode batchNodeWithFile:@"NInvadersResource.pvr.ccz" capacity:100];
    [self addChild:gameBatchNode z:OBJECT_GAME tag:OBJECT_GAME];

    // ボーダーライン
    CCSprite *lineSprite = [CCSprite spriteWithSpriteFrameName:@"line.png"];
    lineSprite.position = ccp(160, GAMEOVER_LINE);
    [gameBatchNode addChild:lineSprite z:SPRITE_GAME_LINE tag:SPRITE_GAME_LINE];
    
    // 床
    CCSprite *yukaSprite = [CCSprite spriteWithSpriteFrameName:@"yuka.png"];
    yukaSprite.anchorPoint = ccp(0.5, 0);
    yukaSprite.position = ccp(160, 50);
    [gameBatchNode addChild:yukaSprite z:SPRITE_GAME_YUKA tag:SPRITE_GAME_YUKA];
}

//---
-(void)gameStart{
    self.mode = MAIN_LAYER_MODE_START;
    
    // スコア初期化
    GameData *gameData = [GameData getInstance];
    [gameData resetScore];

    [[SimpleAudioEngine sharedEngine] stopBackgroundMusic];
    [self animeCountReady];

    // 忍者
    CCSpriteBatchNode *gameBatchNode = (CCSpriteBatchNode*)[self getChildByTag:OBJECT_GAME];
    self.ninja = [[[Ninja alloc] init]autorelease];
    self.ninja.mainScene = (MainScene*)[self parent];
    [gameBatchNode addChild:self.ninja.sprite z:SPRITE_GAME_NINJYA tag:SPRITE_GAME_NINJYA];
}
-(void)animeCountReady{
    CCSpriteBatchNode *gameBatchNode = (CCSpriteBatchNode*)[self getChildByTag:OBJECT_GAME];
    CGSize winSize = [[CCDirector sharedDirector] winSize];
    
    CCSprite *sp;
    sp = [CCSprite spriteWithSpriteFrameName:@"ready.png"];
    sp.position = ccp(160, (winSize.height / 2) + 50);
    sp.opacity = 0;
    [gameBatchNode addChild:sp z:SPRITE_GAME_TEXT];
    
    NSMutableArray* actions = [NSMutableArray array];
    [actions addObject:[CCFadeIn actionWithDuration:0.4f * COUNT_RATE]];
    [actions addObject:[CCDelayTime actionWithDuration:2.6f * COUNT_RATE]];
    [actions addObject:[CCFadeOut actionWithDuration:0.4f * COUNT_RATE]];
    [actions addObject:[CCCallFuncN actionWithTarget:self selector:@selector(animeCountStart:)]];
    
    id action;
    action = [CCSequence actionWithArray:actions];
	[sp runAction:action];
}
-(void)animeCountStart:(id)node{
    [node removeFromParentAndCleanup:YES];

    CCSpriteBatchNode *gameBatchNode = (CCSpriteBatchNode*)[self getChildByTag:OBJECT_GAME];
    CGSize winSize = [[CCDirector sharedDirector] winSize];
    
    CCSprite *sp;
    sp = [CCSprite spriteWithSpriteFrameName:@"start.png"];
    sp.position = ccp(160, (winSize.height / 2) + 50);
    sp.opacity = 0;
    [gameBatchNode addChild:sp z:SPRITE_GAME_TEXT];
    
    NSMutableArray* actions = [NSMutableArray array];
    [actions addObject:[CCFadeIn actionWithDuration:0.4f * COUNT_RATE]];
    [actions addObject:[CCDelayTime actionWithDuration:0.6f * COUNT_RATE]];
    [actions addObject:[CCFadeOut actionWithDuration:0.4f * COUNT_RATE]];
    [actions addObject:[CCCallFuncN actionWithTarget:self selector:@selector(animeCountEnd:)]];
    
    id action;
    action = [CCSequence actionWithArray:actions];
	[sp runAction:action];
}
-(void)animeCountEnd:(id)node{
    [node removeFromParentAndCleanup:YES];
    [[SimpleAudioEngine sharedEngine] playBackgroundMusic:@"game.caf"];
    [self gameInit];
}

//---
-(void)gameInit{
    self.mode = MAIN_LAYER_MODE_PLAY;
}

-(void)gamePlay:(ccTime)dt{
    // 忍者移動
    [self.ninja tick:dt];
}

//---
-(void)gameOver{
    self.mode = MAIN_LAYER_MODE_GAMEOVER;

    [[SimpleAudioEngine sharedEngine] playBackgroundMusic:@"over.caf"];
    
    // ダメージ処理
    NSMutableArray* actions = [NSMutableArray array];
    [actions addObject:[CCShaky3D actionWithRange:8 shakeZ:YES grid:ccg(10,16) duration:0.8]];
                [actions addObject:[CCCallBlock actionWithBlock:^{
        self.grid = nil;
    }]];
    [actions addObject:[CCCallFuncN actionWithTarget:self selector:@selector(animeGameOverStart:)]];
    
    id action = [CCSequence actionWithArray:actions];
	[self runAction:action];
}
-(void)animeGameOverStart:(id)node{
    CCSpriteBatchNode *gameBatchNode = (CCSpriteBatchNode*)[self getChildByTag:OBJECT_GAME];
    CGSize winSize = [[CCDirector sharedDirector] winSize];
    
    CCSprite *sp;
    sp = [CCSprite spriteWithSpriteFrameName:@"gameover.png"];
    sp.position = ccp(160, (winSize.height / 2) + 50);
    sp.opacity = 0;
    [gameBatchNode addChild:sp z:SPRITE_GAME_TEXT];
    
    NSMutableArray* actions = [NSMutableArray array];
    [actions addObject:[CCFadeIn actionWithDuration:0.4f * COUNT_RATE]];
    [actions addObject:[CCDelayTime actionWithDuration:4.6f * COUNT_RATE]];
    [actions addObject:[CCCallFuncN actionWithTarget:self selector:@selector(animeGameOverEnd:)]];
    
    id action;
    action = [CCSequence actionWithArray:actions];
	[sp runAction:action];
}
-(void)animeGameOverEnd:(id)node{
    self.isEndGame = YES;
}

@end