//
//  StickLayer.m
//
//  Created by Kansuke Nishida on 12/07/25.
//  Copyright 2012年 tonosamart. All rights reserved.
//

#import "StickLayer.h"
#import "SimpleAudioEngine.h"
#import "GameData.h"
#import "TonoCocos.h"

enum {
    OBJECT_NULL,
    OBJECT_STICK
};

enum {
    SPRITE_STICK_NULL,
    SPRITE_STICK_L_OFF,
    SPRITE_STICK_L_ON,
    SPRITE_STICK_R_OFF,
    SPRITE_STICK_R_ON
};

@implementation StickLayer

- (id)init{
    self = [[super init] autorelease];
    if (self) {
        // イベントの無効/有効
        self.isTouchEnabled = YES;
        self.isAccelerometerEnabled = NO;
        
        // 変数
        GameData *gameData = [GameData getInstance];
        [gameData resetStickArray];
        
        //レイヤーの初期化
        [self initLayer];
    }
    return self;
}

- (void)dealloc{
    [super dealloc];
}

#pragma mark private

-(void)initLayer{
    [[CCSpriteFrameCache sharedSpriteFrameCache] addSpriteFramesWithFile:@"NInvadersResource.plist"];
    CCSpriteBatchNode *ninvadersBatchNode = [CCSpriteBatchNode batchNodeWithFile:@"NInvadersResource.pvr.ccz" capacity:100];
    [self addChild:ninvadersBatchNode z:OBJECT_STICK tag:OBJECT_STICK];
    
    CCSprite *sp;
    sp = [CCSprite spriteWithSpriteFrameName:@"button_off.png"];
    sp.position = ccp(42, 92);
    [ninvadersBatchNode addChild:sp z:SPRITE_STICK_L_OFF tag:SPRITE_STICK_L_OFF];
    
    sp = [CCSprite spriteWithSpriteFrameName:@"button_on.png"];
    sp.position = ccp(42, 92);
    sp.visible = NO;
    [ninvadersBatchNode addChild:sp z:SPRITE_STICK_L_ON tag:SPRITE_STICK_L_ON];
    
    sp = [CCSprite spriteWithSpriteFrameName:@"button_off.png"];
    sp.position = ccp(320 - 42, 92);
    [ninvadersBatchNode addChild:sp z:SPRITE_STICK_R_OFF tag:SPRITE_STICK_R_OFF];
    
    sp = [CCSprite spriteWithSpriteFrameName:@"button_on.png"];
    sp.position = ccp(320 - 42, 92);
    sp.visible = NO;
    [ninvadersBatchNode addChild:sp z:SPRITE_STICK_R_ON tag:SPRITE_STICK_R_ON];
}

#pragma mark EVENT
-(void)ccTouchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    CCSpriteBatchNode *ninvadersBatchNode = (CCSpriteBatchNode*)[self getChildByTag:OBJECT_STICK];
    GameData *gameData = [GameData getInstance];
	for(UITouch* touch in touches){
		CGPoint touchPoint = [touch locationInView:[touch view]];
		touchPoint = [[CCDirector sharedDirector] convertToGL:touchPoint];

        CCSprite *sp_off_l = (CCSprite*)[ninvadersBatchNode getChildByTag:SPRITE_STICK_L_OFF];
        CCSprite *sp_on_l = (CCSprite*)[ninvadersBatchNode getChildByTag:SPRITE_STICK_L_ON];
		if(CGRectContainsPoint(sp_off_l.boundingBox, touchPoint)){
            [gameData.stickArrayL addObject:touch];
            sp_on_l.visible = YES;
			self.isL = TRUE;
		}
        
        CCSprite *sp_off_r = (CCSprite*)[ninvadersBatchNode getChildByTag:SPRITE_STICK_R_OFF];
        CCSprite *sp_on_r = (CCSprite*)[ninvadersBatchNode getChildByTag:SPRITE_STICK_R_ON];
		if(CGRectContainsPoint(sp_off_r.boundingBox, touchPoint)){
            [gameData.stickArrayR addObject:touch];
            sp_on_r.visible = YES;
			self.isR = TRUE;
		}
	}
}

-(void)ccTouchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
    [self ccTouchesEnded:touches withEvent:event];
    [self ccTouchesBegan:touches withEvent:event];
}

-(void)ccTouchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    CCSpriteBatchNode *ninvadersBatchNode = (CCSpriteBatchNode*)[self getChildByTag:OBJECT_STICK];
    GameData *gameData = [GameData getInstance];

	for(UITouch* touch in touches){
		CGPoint touchPoint = [touch locationInView:[touch view]];
		touchPoint = [[CCDirector sharedDirector] convertToGL:touchPoint];
        
        CCSprite *sp_on_l = (CCSprite*)[ninvadersBatchNode getChildByTag:SPRITE_STICK_L_ON];
		[gameData.stickArrayL removeObject:touch];
		if([gameData.stickArrayL count] == 0){
            sp_on_l.visible = NO;
			self.isL = FALSE;
		}
        
        CCSprite *sp_on_r = (CCSprite*)[ninvadersBatchNode getChildByTag:SPRITE_STICK_R_ON];
		[gameData.stickArrayR removeObject:touch];
		if([gameData.stickArrayR count] == 0){
            sp_on_r.visible = NO;
			self.isR = FALSE;
		}
	}
}
@end